#@+leo
#@+node:0::@file tVector.py
#@+body
#@@language python
#@<< tVector declarations >>
#@+node:1::<< tVector declarations >>
#@+body
import Tkinter
from Tkinter import *
import Pmw


from pdb import set_trace as trace


#@-body
#@-node:1::<< tVector declarations >>


#@+others
#@+node:2::class tVector
#@+body
class tVector:
    """
    Tkinter vectors.

    A helper class for the easy creation and use of Tkinter layouts.

    (don't use this class directly - you should use the derived 'tRow'
    and 'tVer' classes instead)

    For a working example, see this module's docstring
    """

    #@+others
    #@+node:1::__init__
    #@+body
    def __init__(self, direction, *widgets, **kwds):
        """
        Creates an array of widgets (or widget vectors)
        
        Best not to use this class directly - use one of the derived
        classes 'HOR' or 'VER' instead.
        """
        if direction == 'right':
            self.packside = LEFT
        else:
            self.packside = TOP
    
        if kwds.has_key('vectype'):
            self.vectype = kwds['vectype']
            del kwds['vectype']
        else:
            self.vectype = ''
    
        self.kwds = kwds
        self.widlist = widgets
        self.wids = {} # populate this during build
    
    #@-body
    #@-node:1::__init__
    #@+node:2::tBuild
    #@+body
    def tBuild(self, parent):
        """
        Recursively descend through the widgets list, and construct/pack
        the window structure
        """
        self.wids = {}
    
        pack = [BOTH,0,W]
    
        # construct Frame for this vector
        if self.vectype == '':
            #if self.kwds.has_key('panesize'):
            #    self.panesize = self.kwds['panesize']
            #    del self.kwds['panesize']
            #else:
            #    self.panesize = 0
            if getattr(self, 'options', None):
                #print "Creating radio group"
                if self.packside == LEFT:
                    orient='horizontal'
                else:
                    orient='vertical'
                if self.kwds.has_key('font'):
                    font = self.kwds['font']
                    del self.kwds['font']
                else:
                    font = None
                self.topwid = Pmw.RadioSelect(parent,
                                              orient=orient,
                                              buttontype='radiobutton',
                                              **self.kwds)
                self.wids[self.name] = self.topwid
                for opt in self.options:
                    comp = self.topwid.add(opt)
                    if font:
                        comp['font'] = font
                self.topwid.invoke(self.options[self.default])
                # Set the font of the options
                
                #print "widlist =", self.widlist
            else:
                self.topwid = Frame(parent, **self.kwds)
    
        elif self.vectype == 'group':
            self.topwid = Pmw.Group(parent, **self.kwds)
            
        elif self.vectype == 'splitdown':
            if self.kwds.has_key('pack'):
                pack = self.kwds['pack']
                del self.kwds['pack']
                if pack[0] == 0:
                    pack[0] = NONE
            self.topwid = Pmw.PanedWidget(parent,
                                          orient='vertical',
                                          hull_width=self.width,
                                          hull_height=self.height,
                                          **self.kwds)
        elif self.vectype == 'splitright':
            if self.kwds.has_key('pack'):
                pack = self.kwds['pack']
                del self.kwds['pack']
                if pack[0] == 0:
                    pack[0] = NONE
            self.topwid = Pmw.PanedWidget(parent,
                                          orient='horizontal',
                                          **self.kwds)
        self.topwid.pack(side=self.packside,
                         fill=pack[0], expand=pack[1], anchor=pack[2])
        if self.vectype == 'group':
            self.topwid = self.topwid.interior()
       
    
        for wid in self.widlist:
            # create default pack tuple
            pack = [NONE,0,W]
    
            if isinstance(wid, tVector):
                wname = None
    
                if self.vectype in ['splitdown', 'splitright']:
                    if wid.kwds.has_key('panesize'):
                        panesize = wid.kwds['panesize']
                        del wid.kwds['panesize']
                        #print "got vector panesize=%f!" % panesize
                    else:
                        panesize = 0
                    wref = wid.tBuild(self.topwid.add(_newname(),
                                                      size=panesize,
                                                      **self.kwds))
                else:
                    wref = wid.tBuild(self.topwid)
    
                self.wids.update(wid.wids)
                
            else:
                wcls = wid['class']
                wname = wid['name']
                wargs = wid['args']
                wkwds = wid['kwds']
    
                if wkwds.has_key('pack'):
                    pack = wkwds['pack']
                    del wkwds['pack']
                    if pack[0] == 0:
                        pack[0] = NONE
                
                if self.vectype in ['splitdown', 'splitright']:
                    if wkwds.has_key('panesize'):
                        panesize = wkwds['panesize']
                        del wkwds['panesize']
                    else:
                        panesize = 0
                    wref = wcls(self.topwid.add(_newname(),
                                                size=panesize,
                                                **self.kwds),
                                *wargs, **wkwds)
                else:
                    wref = wcls(self.topwid, *wargs, **wkwds)
    
            #print pack
            wref.pack(side=self.packside,
                      fill=pack[0],
                      expand=pack[1],
                      anchor=pack[2])
    
            if wname:
                #print "adding widget '%s' to dict" % wname
                self.wids[wname] = wref
    
        #print "final dict=", self.wids
    
        return self.topwid
    
    #@-body
    #@-node:2::tBuild
    #@+node:3::wid
    #@+body
    def wid(self, widname=None, **kwds):
        """
        Versatile method.
    
        Looks up component widgets by name. If any keywords are supplied,
        the widget's config() method will be called with these keywords.
    
        3 ways of using:
         - with no arguments at all, returns a list of widget names
         - with just a widget name argument, returns a ref to that widget
         - with widget name and some keywords, calls config() on that widget
           and passes the given keywords.
    
        Examples:
    
          # get a list of widgets for vector 'mywin'
          widgets = mywin.wid()
    
          # get a handle to the widget 'label1'
          mylabel = mywin.wid('label1')
    
          # change the text of 'label1'
          mywin.wid('label1', text="Different text for label1")
    
        """
        if not widname:
            return self.wids.keys()
        else:
            try:
                wid = self.wids[widname]
                if len(kwds) > 0:
                    wid.config(**kwds)
                else:
                    return self.wids[widname]
            except:
                raise AttributeError
    
    #@-body
    #@-node:3::wid
    #@-others


#@-body
#@-node:2::class tVector
#@+node:3::class tRow
#@+body

class tRow(tVector):
    """
    Creates a horizontal vector of widgets.

    For usage info, refer to tVector docstring
    """

    #@+others
    #@+node:1::__init__
    #@+body
    def __init__(self, *widgets, **kwds):
        """
        Constructor for tRow - a horizontal vector of widgets
    
        Arguments:
         - widget1
         - widget2
         - ...
    
        Each widget should be either be something returned from tW,
        or a tRow or tCol instance.
    
        For an example, refer to docstring for tVector class
        """
        tVector.__init__(self, 'right', *widgets, **kwds)
    
    #@-body
    #@-node:1::__init__
    #@-others


#@-body
#@-node:3::class tRow
#@+node:4::class tCol
#@+body

class tCol(tVector):
    """
    Creates a vertical column of widgets.

    For usage info, refer to tVector docstring
    """

    #@+others
    #@+node:1::__init__
    #@+body
    def __init__(self, *widgets, **kwds):
        """
        Constructor for tRow - a vertical vector of widgets
    
        Arguments:
         - direction - 'across' or 'down'
         - options - list of strings
         - default - default option index
    
        Each widget should be either be something returned from tW,
        or a tRow or tCol instance.
    
        For an example, refer to docstring for tVector class
        """
        # tVector.tBuild will act correctly on this group
        tVector.__init__(self, 'down', *widgets, **kwds)
    
    #@-body
    #@-node:1::__init__
    #@-others


#@-body
#@-node:4::class tCol
#@+node:5::class tColSplit
#@+body

class tColSplit(tCol):
    """
    Creates a splitter column

    Note - there will be one splitter pane created for each widget.
    If you want more than one widget in a frame, then put these
    widgets into a nested tCol or tRow object

    Construction is same as tCol, except that the first 2 args must
    be the width and height of the aplitter hull
    """


    #@+others
    #@+node:1::__init__
    #@+body
    def __init__(self, width, height, *widgets, **kwds):
        """
        Constructs a split column
        """
        # tVector.tBuild will act correctly on the splitter
        kwds['vectype'] = 'splitdown'
        self.width = width
        self.height = height
        tCol.__init__(self, *widgets, **kwds)
    
    #@-body
    #@-node:1::__init__
    #@-others


#@-body
#@-node:5::class tColSplit
#@+node:6::class tRowSplit
#@+body

class tRowSplit(tRow):
    """
    Creates a splitter row

    Note - there will be one splitter pane created for each widget.
    If you want more than one widget in a frame, then put these
    widgets into a nested tCol or tRow object

    Construction is same as tCol, except that the first 2 args must
    be the width and height of the aplitter hull
    """

    #@+others
    #@+node:1::__init__
    #@+body
    def __init__(self, width, height, *widgets, **kwds):
        """
        Construct a splitter row
        """
        # tVector.tBuild will act correctly on the splitter
        kwds['vectype'] = 'splitright'
        self.width = width
        self.height = height
        tRow.__init__(self, *widgets, **kwds)
    
    #@-body
    #@-node:1::__init__
    #@-others


#@-body
#@-node:6::class tRowSplit
#@+node:7::class tGroup
#@+body

class tGroup(tCol):
    """
    Creates a group

    Construction is same as tCol.
    If you want to arrange stuff horizontally within the group, you might
    want to embed one or more tRow objects
    """


    #@+others
    #@+node:1::__init__
    #@+body
    def __init__(self, *widgets, **kwds):
        """
        Constructs a group column
        """
        # tVector.tBuild will act correctly on this group
        kwds['vectype'] = 'group'
        tCol.__init__(self, *widgets, **kwds)
    
    #@-body
    #@-node:1::__init__
    #@-others


#@-body
#@-node:7::class tGroup
#@+node:8::class tRadio
#@+body

class tRadio(tVector):
    """
    Creates a set of radiobuttons
    """


    #@+others
    #@+node:1::__init__
    #@+body
    def __init__(self, name, direction, options, default, **kwds):
        """
        Constructs a set of radiobuttons
        """
        tVector.__init__(self, direction, **kwds)
        self.name = name
        self.options = options
        self.default = default
    
    #@-body
    #@-node:1::__init__
    #@-others


#@-body
#@-node:8::class tRadio
#@+node:9::tW
#@+body
def tW(widclass, widname, *args, **kwds):
    """
    Helper function for tRow and tCol classes.
    
    Generates a widget entry and translates it into a dict that can be
    fed straight to the tRow and tCol constructors.

    Refer to the tVector docstring for more info.

    Arguments:
     - widclass - a widget class object that behaves like normal tk widgets
     - widname - name to give to this widget (accessible later via
       tRow.wid and tCol.wid methods)
     - extra arguments are passed to the widget constructor
    Keywords:
     - anything you give gets passed straight to widget constructor

    Example:

       tW(Tkinter.Button, 'mybutton1', text="Button 1")
    """
                
    return {'name':widname,
            'class':widclass,
            'args':args,
            'kwds':kwds}

#@-body
#@-node:9::tW
#@+node:10::_newname
#@+body
_newnum = 0

def _newname():
    global _newnum
    name = "pane%d" % _newnum
    _newnum += 1
    return name

#@-body
#@-node:10::_newname
#@-others


"""
Convenience layer for Tkinter.

Allows window specifications to be created with far less,
and much more readable code, than doing it by hand.

Contains classes tRow and tCol, for building rows and columns
of widgets, and building these into a live frame.

Example:

    from Tkinter import *
    from tVector import *
    
    class mywin:
        def __init__(self, parent):
            self.layout = tCol(tW(Label, 'L1',
                                  text="Window Heading",
                                  font="helvetica 16 bold"),
                               tW(Checkbutton, 'Chk1',
                                  text="A checkbutton"),
                               tRow(tW(Button, 'but1',
                                       text="Button 1",
                                       command=self.on_button1),
                                    tW(Button, 'but2',
                                       text="Button 2",
                                       command=self.on_button2),
                                    tW(Button, 'but3',
                                       text="Button 3",
                                       command=self.on_button3)),
                               tRow(tW(Entry, 'fld1',
                                       text="field 1"),
                                    tW(Entry, 'fld2',
                                       text="field 2")))
            self.layout.tBuild(parent)
    
        def on_button1(self):
            print "clicked button1"
            self.layout.wid('but1', text="New Button1 label")
    
        def on_button2(self):
            print "clicked button2"
            self.layout.wid('but1', text='Button 1')
    
        def on_button3(self):
            print "clicked button3"
    
        def mainloop(self):
            self.layout.mainloop()
        
    root = Tk()
    app = mywin(root)
    app.mainloop()

"""


#@-body
#@-node:0::@file tVector.py
#@-leo
